/************************* RANROTW.CPP ****************** AgF 1999-03-03 *
*  Random Number generator 'RANROT' type W                               *
*  This version is used when a resolution higher that 32 bits is desired.*
*                                                                        *
*  This is a lagged-Fibonacci type of random number generator with       *
*  rotation of bits.  The algorithm is:                                  *
*  Z[n] = (Y[n-j] + (Y[n-k] rotl r1)) modulo 2^(b/2)                     *
*  Y[n] = (Z[n-j] + (Z[n-k] rotl r2)) modulo 2^(b/2)                     *
*  X[n] = Y[n] + Z[n]*2^(b/2)                                            *
*                                                                        *
*  The last k values of Y and Z are stored in a circular buffer named    *
*  randbuffer.                                                           *
*  The code includes a self-test facility which will detect any          *
*  repetition of previous states.                                        *
*  The function uses a fast method for conversion to floating point.     *
*  This method relies on floating point numbers being stored in the      *
*  standard 64-bit IEEE format or the 80-bit long double format.         *
*                                                                        *
*  The theory of the RANROT type of generators and the reason for the    *
*  self-test are described at www.agner.org/random/ranrot.htm            *
*                                                                        *
*************************************************************************/

// names of header files may differ
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <mem.h>
#include <stdlib.h>
#include <assert.h>

// define if you want self-test:
#define SELF_TEST
// define if long double resolution desired and available:
// #define HIGH_RESOLUTION

// define the types used:
typedef unsigned long uint32;       // use 32 bit unsigned integers
#ifdef HIGH_RESOLUTION
typedef long double trfloat;        // define floating point precision
#else
typedef double trfloat;             // define floating point precision
#endif

// If your system doesn't have a rotate function for 32 bits integers,
// then define it thus:
// uint32 _lrotl (uint32 x, int r) {
//   return (x << r) | (x >> (sizeof(x)*8-r));}

// define parameters
const int KK = 17;
const int JJ = 10;
const int R1 = 19;
const int R2 = 27;


class TRanrotWGenerator {           // encapsulate random number generator
  public:
  void RandomInit(uint32 seed);     // initialization
  void SetInterval(int min, int max); // set interval for iRandom
  int iRandom();                    // get integer random number
  trfloat Random();                 // get floating point random number
  TRanrotWGenerator(uint32 seed=-1); // constructor
  protected:
  void step();                      // generate next random number
  union {                           // used for conversion to float
    trfloat randp1;
    uint32 randbits[3];};
  int p1, p2;                       // indexes into buffer
  int imin, iinterval;              // interval for iRandom
  uint32 randbuffer[KK][2];         // history buffer
#ifdef SELF_TEST
  uint32 randbufcopy[KK*2][2];      // used for self-test
#endif
};


// constructor:
TRanrotWGenerator::TRanrotWGenerator(uint32 seed) {
  RandomInit(seed);  SetInterval(0, 99);}


void TRanrotWGenerator::step() {
  // generate next random number
  uint32 y, z;
  // generate next number
  z = _lrotl(randbuffer[p1][0], R1) + randbuffer[p2][0];
  y = _lrotl(randbuffer[p1][1], R2) + randbuffer[p2][1];
  randbuffer[p1][0] = y; randbuffer[p1][1] = z;
  // rotate list pointers
  if (--p1 < 0) p1 = KK - 1;
  if (--p2 < 0) p2 = KK - 1;
#ifdef SELF_TEST
  // perform self-test
  if (randbuffer[p1][0] == randbufcopy[0][0] &&
    memcmp(randbuffer, randbufcopy[KK-p1], 2*KK*sizeof(uint32)) == 0) {
      // self-test failed
      if ((p2 + KK - p1) % KK != JJ) {
        // note: the way of printing error messages depends on system
        // In Windows you may use FatalAppExit
        printf("Random number generator not initialized");}
      else {
        printf("Random number generator returned to initial state");}
      exit(1);}
#endif
  // fast conversion to float:
  randbits[0] = y;
#ifdef HIGH_RESOLUTION            // 80 bits floats = 63 bits resolution
  randbits[1] = z | 0x80000000;
#else                             // 64 bits floats = 52 bits resolution
  randbits[1] = (z & 0x000FFFFF) | 0x3FF00000;
#endif
  }


trfloat TRanrotWGenerator::Random() {
  // returns a random number between 0 and 1.
  trfloat r = randp1 - 1.;
  step();
  return r;}


int TRanrotWGenerator::iRandom() {
  // get integer random number
  int i = iinterval * Random();
  if (i >= iinterval) i = iinterval;
  return imin + i;}


// set interval for iRandom
void TRanrotWGenerator::SetInterval(int min, int max) {
  imin = min; iinterval = max - min + 1;}


void TRanrotWGenerator::RandomInit (uint32 seed) {
  // this function initializes the random number generator.
  int i, j;
  // make sure seed != 0
  if (seed==0) seed--;

  // make random numbers and put them into the buffer
  for (i=0; i<KK; i++) {
    for (j=0; j<2; j++) {
      seed ^= seed << 13; seed ^= seed >> 17; seed ^= seed << 5;
      randbuffer[i][j] = seed;}}
  // set exponent of randp1
  randp1 = 1.5;
#ifdef HIGH_RESOLUTION
  assert((randbits[2]&0xFFFF)==0x3FFF); // check that Intel 10-byte float format used
#else
  assert(randbits[1]==0x3FF80000); // check that IEEE double precision float format used
#endif
  assert(sizeof(uint32)==4);  // check that 32 bits integers used

  // initialize pointers to circular buffer
  p1 = 0;  p2 = JJ;
#ifdef SELF_TEST
  // store state for self-test
  memcpy (randbufcopy, randbuffer, 2*KK*sizeof(uint32));
  memcpy (randbufcopy[KK], randbuffer, 2*KK*sizeof(uint32));
#endif
  // randomize some more
  for (i=0; i<97; i++) step();
}


//////////////////////////////////////////////////////////////////////////
// Example showing how to use the random number generator:
void main() {
  TRanrotWGenerator RG;   // define random number generator

  RG.RandomInit(time(0)); // use time as random seed

  RG.SetInterval(1, 10);  // set interval for integer output

  for (int i=0; i<20; i++) {
    // generate 10 random floating point numbers and 10 random integers
    printf("\n%14.10f   %2i", (double)RG.Random(), RG.iRandom());}

  getch();  // wait for user to press any key
  }
